"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var widgets_1 = require("@phosphor/widgets");
var apputils_1 = require("@jupyterlab/apputils");
var mainmenu_1 = require("@jupyterlab/mainmenu");
var codeeditor_1 = require("@jupyterlab/codeeditor");
var codemirror_1 = require("@jupyterlab/codemirror");
var coreutils_1 = require("@jupyterlab/coreutils");
var fileeditor_1 = require("@jupyterlab/fileeditor");
/**
 * The command IDs used by the codemirror plugin.
 */
var CommandIDs;
(function (CommandIDs) {
    CommandIDs.changeKeyMap = 'codemirror:change-keymap';
    CommandIDs.changeTheme = 'codemirror:change-theme';
    CommandIDs.changeMode = 'codemirror:change-mode';
    CommandIDs.find = 'codemirror:find';
    CommandIDs.findAndReplace = 'codemirror:find-and-replace';
})(CommandIDs || (CommandIDs = {}));
/**
 * The editor services.
 */
var services = {
    id: '@jupyterlab/codemirror-extension:services',
    provides: codeeditor_1.IEditorServices,
    activate: function () { return codemirror_1.editorServices; }
};
/**
 * The editor commands.
 */
var commands = {
    id: '@jupyterlab/codemirror-extension:commands',
    requires: [
        fileeditor_1.IEditorTracker,
        mainmenu_1.IMainMenu,
        apputils_1.ICommandPalette,
        coreutils_1.IStateDB,
        coreutils_1.ISettingRegistry
    ],
    activate: activateEditorCommands,
    autoStart: true
};
/**
 * Export the plugins as default.
 */
var plugins = [commands, services];
exports.default = plugins;
/**
 * The plugin ID used as the key in the setting registry.
 */
var id = commands.id;
/**
 * Set up the editor widget menu and commands.
 */
function activateEditorCommands(app, tracker, mainMenu, palette, state, settingRegistry) {
    var commands = app.commands, restored = app.restored;
    var _a = codemirror_1.CodeMirrorEditor.defaultConfig, theme = _a.theme, keyMap = _a.keyMap;
    /**
     * Update the setting values.
     */
    function updateSettings(settings) {
        keyMap = settings.get('keyMap').composite || keyMap;
        theme = settings.get('theme').composite || theme;
    }
    /**
     * Update the settings of the current tracker instances.
     */
    function updateTracker() {
        tracker.forEach(function (widget) {
            if (widget.editor instanceof codemirror_1.CodeMirrorEditor) {
                var cm = widget.editor.editor;
                cm.setOption('keyMap', keyMap);
                cm.setOption('theme', theme);
            }
        });
    }
    // Fetch the initial state of the settings.
    Promise.all([settingRegistry.load(id), restored]).then(function (_a) {
        var settings = _a[0];
        updateSettings(settings);
        updateTracker();
        settings.changed.connect(function () {
            updateSettings(settings);
            updateTracker();
        });
    }).catch(function (reason) {
        console.error(reason.message);
        updateTracker();
    });
    /**
     * Handle the settings of new widgets.
     */
    tracker.widgetAdded.connect(function (sender, widget) {
        if (widget.editor instanceof codemirror_1.CodeMirrorEditor) {
            var cm = widget.editor.editor;
            cm.setOption('keyMap', keyMap);
            cm.setOption('theme', theme);
        }
    });
    /**
     * A test for whether the tracker has an active widget.
     */
    function isEnabled() {
        return tracker.currentWidget !== null &&
            tracker.currentWidget === app.shell.currentWidget;
    }
    /**
     * Create a menu for the editor.
     */
    function createMenu() {
        var menu = new widgets_1.Menu({ commands: commands });
        var themeMenu = new widgets_1.Menu({ commands: commands });
        var keyMapMenu = new widgets_1.Menu({ commands: commands });
        var modeMenu = new widgets_1.Menu({ commands: commands });
        var tabMenu = new widgets_1.Menu({ commands: commands });
        menu.title.label = 'Editor';
        themeMenu.title.label = 'Theme';
        keyMapMenu.title.label = 'Key Map';
        modeMenu.title.label = 'Language';
        tabMenu.title.label = 'Tabs';
        commands.addCommand(CommandIDs.changeTheme, {
            label: function (args) { return args['theme']; },
            execute: function (args) {
                var key = 'theme';
                var value = theme = args['theme'] || theme;
                updateTracker();
                return settingRegistry.set(id, key, value).catch(function (reason) {
                    console.error("Failed to set " + id + ":" + key + " - " + reason.message);
                });
            },
            isEnabled: isEnabled,
            isToggled: function (args) { return args['theme'] === theme; }
        });
        commands.addCommand(CommandIDs.changeKeyMap, {
            label: function (args) {
                var title = args['keyMap'];
                return title === 'sublime' ? 'Sublime Text' : title;
            },
            execute: function (args) {
                var key = 'keyMap';
                var value = keyMap = args['keyMap'] || keyMap;
                updateTracker();
                return settingRegistry.set(id, key, value).catch(function (reason) {
                    console.error("Failed to set " + id + ":" + key + " - " + reason.message);
                });
            },
            isEnabled: isEnabled,
            isToggled: function (args) { return args['keyMap'] === keyMap; }
        });
        commands.addCommand(CommandIDs.find, {
            label: 'Find...',
            execute: function () {
                var widget = tracker.currentWidget;
                if (!widget) {
                    return;
                }
                var editor = widget.editor;
                editor.execCommand('find');
            },
            isEnabled: isEnabled
        });
        commands.addCommand(CommandIDs.findAndReplace, {
            label: 'Find & Replace...',
            execute: function () {
                var widget = tracker.currentWidget;
                if (!widget) {
                    return;
                }
                var editor = widget.editor;
                editor.execCommand('replace');
            },
            isEnabled: isEnabled
        });
        commands.addCommand(CommandIDs.changeMode, {
            label: function (args) { return args['name']; },
            execute: function (args) {
                var name = args['name'];
                var widget = tracker.currentWidget;
                if (name && widget) {
                    var spec = codemirror_1.Mode.findByName(name);
                    if (spec) {
                        widget.model.mimeType = spec.mime;
                    }
                }
            },
            isEnabled: isEnabled,
            isToggled: function (args) {
                var widget = tracker.currentWidget;
                if (!widget) {
                    return false;
                }
                var mime = widget.model.mimeType;
                var spec = codemirror_1.Mode.findByMIME(mime);
                var name = spec && spec.name;
                return args['name'] === name;
            }
        });
        codemirror_1.Mode.getModeInfo().sort(function (a, b) {
            var aName = a.name || '';
            var bName = b.name || '';
            return aName.localeCompare(bName);
        }).forEach(function (spec) {
            // Avoid mode name with a curse word.
            if (spec.mode.indexOf('brainf') === 0) {
                return;
            }
            modeMenu.addItem({
                command: CommandIDs.changeMode,
                args: __assign({}, spec)
            });
        });
        [
            'jupyter', 'default', 'abcdef', 'base16-dark', 'base16-light',
            'hopscotch', 'material', 'mbo', 'mdn-like', 'seti', 'the-matrix',
            'xq-light', 'zenburn'
        ].forEach(function (name) { return themeMenu.addItem({
            command: CommandIDs.changeTheme,
            args: { theme: name }
        }); });
        ['default', 'sublime', 'vim', 'emacs'].forEach(function (name) {
            keyMapMenu.addItem({
                command: CommandIDs.changeKeyMap,
                args: { keyMap: name }
            });
        });
        var args = {
            insertSpaces: false, size: 4, name: 'Indent with Tab'
        };
        var command = 'fileeditor:change-tabs';
        tabMenu.addItem({ command: command, args: args });
        palette.addItem({ command: command, args: args, category: 'Editor' });
        for (var _i = 0, _a = [1, 2, 4, 8]; _i < _a.length; _i++) {
            var size = _a[_i];
            var args_1 = {
                insertSpaces: true, size: size, name: "Spaces: " + size + " "
            };
            tabMenu.addItem({ command: command, args: args_1 });
            palette.addItem({ command: command, args: args_1, category: 'Editor' });
        }
        menu.addItem({ command: 'fileeditor:toggle-autoclosing-brackets' });
        menu.addItem({ type: 'submenu', submenu: tabMenu });
        menu.addItem({ type: 'separator' });
        menu.addItem({ type: 'submenu', submenu: modeMenu });
        menu.addItem({ type: 'submenu', submenu: keyMapMenu });
        menu.addItem({ type: 'submenu', submenu: themeMenu });
        return menu;
    }
    mainMenu.addMenu(createMenu(), { rank: 30 });
    // Add find-replace capabilities to the edit menu.
    mainMenu.editMenu.findReplacers.add({
        tracker: tracker,
        find: function (widget) {
            var editor = widget.editor;
            editor.execCommand('find');
        },
        findAndReplace: function (widget) {
            var editor = widget.editor;
            editor.execCommand('replace');
        }
    });
}
